@@ -3,18 +3,20 @@ from .itertoolz import (groupby, countby, frequencies, reduceby,
                         get, merge_sorted, concat, concatv, mapcat,
                         isdistinct, interleave, unique, intersection,
                         isiterable, remove, iterate, accumulate,
-                        partitionby)
+                        partitionby, sliding_window, count)
 
 from .functoolz import (memoize, curry, compose, thread_first,
-                        thread_last, identity)
+                        thread_last, identity, pipe)
 
-from .dicttoolz import merge, keymap, valmap, assoc, update_in
+from .dicttoolz import merge, merge_with, keymap, valmap, assoc, update_in
 
 from .compatibility import map, filter
 
 from functools import partial, reduce
 
+sorted = sorted
+
 # Aliases
 comp = compose
 
-__version__ = '0.3.0'
+__version__ = '0.4.0'
