@@ -1,5 +1,6 @@
-from functools import reduce
+from functools import reduce, partial
 import itertools
+import inspect
 
 
 def identity(x):
@@ -19,7 +20,7 @@ def thread_first(val, *forms):
 
     >>> def add(x, y): return x + y
     >>> def pow(x, y): return x**y
-    >>> thread_first(1, (add, 4), (pow, 2))  # pow(add(4, 1), 2)
+    >>> thread_first(1, (add, 4), (pow, 2))  # pow(add(1, 4), 2)
     25
 
     So in general
@@ -53,7 +54,7 @@ def thread_last(val, *forms):
 
     >>> def add(x, y): return x + y
     >>> def pow(x, y): return x**y
-    >>> thread_last(1, (add, 4), (pow, 2))  # pow(2, add(1, 4))
+    >>> thread_last(1, (add, 4), (pow, 2))  # pow(2, add(4, 1))
     32
 
     So in general
@@ -61,8 +62,9 @@ def thread_last(val, *forms):
     expands to
         g(y, z, f(x))
 
-    >>> def even(x): return x % 2 == 0
-    >>> list(thread_last([1, 2, 3], (map, inc), (filter, even)))
+    >>> def iseven(x):
+    ...     return x % 2 == 0
+    >>> list(thread_last([1, 2, 3], (map, inc), (filter, iseven)))
     [2, 4]
 
     See Also:
@@ -102,13 +104,16 @@ def memoize(f, cache=None):
     ... def add(x, y):
     ...     return x + y
     """
-    if cache == None:
+    if cache is None:
         cache = {}
 
     def memof(*args):
-        if not hashable(args):
-            return f(*args)
-        elif args in cache:
+        try:
+            in_cache = args in cache
+        except TypeError:
+            raise TypeError("Arguments to memoized function must be hashable")
+
+        if in_cache:
             return cache[args]
         else:
             result = f(*args)
@@ -119,6 +124,31 @@ def memoize(f, cache=None):
     return memof
 
 
+def _num_required_args(func):
+    """ Number of args for func
+
+    >>> def foo(a, b, c=None):
+    ...     return a + b + c
+
+    >>> _num_required_args(foo)
+    2
+
+    >>> def bar(*args):
+    ...     return sum(args)
+
+    >>> print(_num_required_args(bar))
+    None
+    """
+    try:
+        spec = inspect.getargspec(func)
+        if spec.varargs:
+            return None
+        num_defaults = len(spec.defaults) if spec.defaults else 0
+        return len(spec.args) - num_defaults
+    except TypeError:
+        return None
+
+
 class curry(object):
     """ Curry a callable function
 
@@ -142,11 +172,15 @@ class curry(object):
     >>> add = f(a=1)
     >>> add(2, 3)
     5
+
+    See Also:
+        toolz.curried - namespace of curried functions
+                        http://toolz.readthedocs.org/en/latest/curry.html
     """
     def __init__(self, func, *args, **kwargs):
         self.func = func
         self.args = args
-        self.kwargs = kwargs
+        self.keywords = kwargs if kwargs else None
         self.__doc__ = self.func.__doc__
         try:
             self.func_name = self.func.func_name
@@ -161,12 +195,32 @@ class curry(object):
 
     def __call__(self, *args, **_kwargs):
         args = self.args + args
-        kwargs = {}
-        kwargs.update(self.kwargs)
-        kwargs.update(_kwargs)
+        if _kwargs:
+            kwargs = {}
+            if self.keywords:
+                kwargs.update(self.keywords)
+            kwargs.update(_kwargs)
+        elif self.keywords:
+            kwargs = self.keywords
+        else:
+            kwargs = {}
+
         try:
             return self.func(*args, **kwargs)
-        except TypeError:
+        except TypeError as e:
+            required_args = _num_required_args(self.func)
+
+            # If there was a genuine TypeError
+            if required_args is not None and len(args) >= required_args:
+                raise e
+
+            # If we only need one more argument
+            if (required_args is not None and required_args - len(args) == 1):
+                if kwargs:
+                    return partial(self.func, *args, **kwargs)
+                else:
+                    return partial(self.func, *args)
+
             return curry(self.func, *args, **kwargs)
 
 
@@ -183,6 +237,9 @@ def compose(*funcs):
     >>> inc = lambda i: i + 1
     >>> compose(str, inc)(3)
     '4'
+
+    See Also:
+        pipe
     """
     if not funcs:
         return identity
@@ -198,3 +255,27 @@ def compose(*funcs):
             return ret
 
         return composed
+
+
+def pipe(data, *functions):
+    """ Pipe a value through a sequence of functions
+
+    I.e. ``pipe(data, f, g, h)`` is equivalent to ``h(g(f(data)))``
+
+    We think of the value as progressing through a pipe of several
+    transformations, much like pipes in UNIX
+
+    ``$ cat data | f | g | h``
+
+    >>> double = lambda i: 2 * i
+    >>> pipe(3, double, str)
+    '6'
+
+    See Also:
+        compose
+        thread_first
+        thread_last
+    """
+    for func in functions:
+        data = func(data)
+    return data
