@@ -2,6 +2,7 @@ from .core import (groupby, remove, concat, concatv, mapcat,
                    frequencies, interpose, first, second, nth, take,
                    drop, rest, get, last, merge_sorted, interleave,
                    unique, reduceby, intersection, isiterable,
-                   isdistinct, cons, iterate, accumulate)
+                   isdistinct, cons, iterate, accumulate, sliding_window,
+                   count)
 
 from .recipes import countby, partitionby
