@@ -1,6 +1,8 @@
+import heapq
 import itertools
 from functools import partial
-from toolz.compatibility import Queue, map
+from toolz.compatibility import map
+import collections
 
 
 identity = lambda x: x
@@ -9,15 +11,15 @@ identity = lambda x: x
 def remove(predicate, coll):
     """ Return those items of collection for which predicate(item) is true.
 
-    >>> def even(x):
+    >>> def iseven(x):
     ...     return x % 2 == 0
-    >>> list(remove(even, [1, 2, 3, 4]))
+    >>> list(remove(iseven, [1, 2, 3, 4]))
     [1, 3]
     """
     return filter(lambda x: not predicate(x), coll)
 
 
-def accumulate(f, seq):
+def accumulate(binop, seq):
     """ Repeatedly apply binary function f to a sequence, accumulating results
 
     >>> from operator import add, mul
@@ -39,7 +41,7 @@ def accumulate(f, seq):
     result = next(iter(seq))
     yield result
     for elem in itertools.islice(seq, 1, None):
-        result = f(result, elem)
+        result = binop(result, elem)
         yield result
 
 
@@ -49,6 +51,13 @@ def groupby(f, coll):
     >>> names = ['Alice', 'Bob', 'Charlie', 'Dan', 'Edith', 'Frank']
     >>> groupby(len, names)
     {3: ['Bob', 'Dan'], 5: ['Alice', 'Edith', 'Frank'], 7: ['Charlie']}
+
+    >>> iseven = lambda x: x % 2 == 0
+    >>> groupby(iseven, [1, 2, 3, 4, 5, 6, 7, 8])
+    {False: [1, 3, 5, 7], True: [2, 4, 6, 8]}
+
+    See Also:
+        ``countby``
     """
     d = dict()
     for item in coll:
@@ -62,32 +71,59 @@ def groupby(f, coll):
 def merge_sorted(*iters, **kwargs):
     """ Merge and sort a collection of sorted collections
 
+    This works lazily and only keeps one value from each iterable in memory.
+
     >>> list(merge_sorted([1, 3, 5], [2, 4, 6]))
     [1, 2, 3, 4, 5, 6]
 
     >>> ''.join(merge_sorted('abc', 'abc', 'abc'))
     'aaabbbccc'
+
+    The "key" function used to sort the input may be passed as a keyword.
+
+    >>> list(merge_sorted([2, 3], [1, 3], key=lambda x: x // 3))
+    [2, 1, 3, 3]
     """
-    key = kwargs.get('key', identity)
-    iters = map(iter, iters)
-    pq = Queue.PriorityQueue()
+    key = kwargs.get('key', None)
+    if key is None:
+        # heapq.merge does what we do below except by val instead of key(val)
+        for item in heapq.merge(*iters):
+            yield item
+    else:
+        # The commented code below shows an alternative (slower) implementation
+        # to apply a key function for sorting.
+        #
+        # mapper = lambda i, item: (key(item), i, item)
+        # keyiters = [map(partial(mapper, i), itr) for i, itr in enumerate(iters)]
+        # return (item for (item_key, i, item) in heapq.merge(*keyiters))
 
-    def inject_first_element(it, tiebreaker=None):
-        try:
-            item = next(it)
-            pq.put((key(item), item, tiebreaker, it))
-        except StopIteration:
-            pass
+        # binary heap as a priority queue
+        pq = []
 
-    # Initial population
-    for i, it in enumerate(iters):
-        inject_first_element(it, i)
+        # Initial population
+        for itnum, it in enumerate(map(iter, iters)):
+            try:
+                item = next(it)
+                pq.append([key(item), itnum, item, it])
+            except StopIteration:
+                pass
+        heapq.heapify(pq)
 
-    # Repeatedly yield and then repopulate from the same iterator
-    while not pq.empty():
-        _, item, tb, it = pq.get()
-        yield item
-        inject_first_element(it, tb)
+        # Repeatedly yield and then repopulate from the same iterator
+        while True:
+            try:
+                while True:
+                    # raises IndexError when pq is empty
+                    _, itnum, item, it = s = pq[0]
+                    yield item
+                    item = next(it)  # raises StopIteration when exhausted
+                    s[0] = key(item)
+                    s[2] = item
+                    heapq.heapreplace(pq, s)  # restore heap condition
+            except StopIteration:
+                heapq.heappop(pq)  # remove empty iterator
+            except IndexError:
+                return
 
 
 def interleave(seqs, pass_exceptions=()):
@@ -206,6 +242,14 @@ def first(seq):
     return next(iter(seq))
 
 
+def second(seq):
+    """ The second element in a sequence
+
+    >>> second('ABC')
+    'B'
+    """
+    return next(itertools.islice(seq, 1, None))
+
 def nth(n, seq):
     """ The nth element in a sequence
 
@@ -226,7 +270,7 @@ def last(seq):
     """
     try:
         return seq[-1]
-    except TypeError:
+    except (TypeError, KeyError):
         old = None
         it = iter(seq)
         while True:
@@ -236,12 +280,17 @@ def last(seq):
                 return old
 
 
-second = partial(nth, 1)
 rest = partial(drop, 1)
 
 
 no_default = '__no__default__'
 
+def _get(ind, seq, default):
+    try:
+        return seq[ind]
+    except (KeyError, IndexError):
+        return default
+
 
 def get(ind, seq, default=no_default):
     """ Get element in a sequence or dict
@@ -273,14 +322,22 @@ def get(ind, seq, default=no_default):
     >>> get(['Alice', 'Dennis'], phonebook, None)
     ('555-1234', None)
     """
-    if isinstance(ind, list):
-        return tuple(get(i, seq, default) for i in ind)
-    if default is no_default:
+    try:
         return seq[ind]
-    else:
-        try:
-            return seq[ind]
-        except (KeyError, IndexError):
+    except TypeError:  # `ind` may be a list
+        if isinstance(ind, list):
+            if default is no_default:
+                return tuple(seq[i] for i in ind)
+            else:
+                return tuple(_get(i, seq, default) for i in ind)
+        elif default is not no_default:
+            return default
+        else:
+            raise
+    except (KeyError, IndexError) as e:  # we know `ind` is not a list
+        if default is no_default:
+            raise e
+        else:
             return default
 
 
@@ -367,15 +424,16 @@ def reduceby(keyfn, binop, seq, init):
     """ Perform a simultaneous groupby and reduction
 
     The computation:
-        result = reduceby(keyfn, binop, seq, init)
+
+    >>> result = reduceby(keyfn, binop, seq, init)      # doctest: +SKIP
 
     is equivalent to the following:
-        groups = groupby(keyfn, seq)
 
-        def reduction(group):
-            return reduce(binop, group, init)
+    >>> def reduction(group):                           # doctest: +SKIP
+    ...     return reduce(binop, group, init)           # doctest: +SKIP
 
-        result = {k: reduction(group) for k, group in groups.items()}
+    >>> groups = groupby(keyfn, seq)                    # doctest: +SKIP
+    >>> result = valmap(reduction, groups)              # doctest: +SKIP
 
     But the former does not build the intermediate groups, allowing it to
     operate in much less space.  This makes it suitable for larger datasets
@@ -383,10 +441,10 @@ def reduceby(keyfn, binop, seq, init):
 
     >>> from operator import add, mul
     >>> data = [1, 2, 3, 4, 5]
-    >>> is_even = lambda x: x % 2 == 0
-    >>> reduceby(is_even, add, data, 0)
+    >>> iseven = lambda x: x % 2 == 0
+    >>> reduceby(iseven, add, data, 0)
     {False: 9, True: 6}
-    >>> reduceby(is_even, mul, data, 1)
+    >>> reduceby(iseven, mul, data, 1)
     {False: 15, True: 8}
 
     >>> projects = [{'name': 'build roads', 'state': 'CA', 'cost': 1000000},
@@ -420,7 +478,54 @@ def iterate(f, x):
     1
     >>> next(counter)
     2
+
+    >>> double = lambda x: x * 2
+    >>> powers_of_two = iterate(double, 1)
+    >>> next(powers_of_two)
+    1
+    >>> next(powers_of_two)
+    2
+    >>> next(powers_of_two)
+    4
+    >>> next(powers_of_two)
+    8
+
     """
     while True:
         yield x
         x = f(x)
+
+
+def sliding_window(n, seq):
+    """ A sequence of overlapping subsequences
+
+    >>> list(sliding_window(2, [1, 2, 3, 4]))
+    [(1, 2), (2, 3), (3, 4)]
+
+    This function creates a sliding window suitable for transformations like
+    sliding means / smoothing
+
+    >>> mean = lambda seq: float(sum(seq)) / len(seq)
+    >>> list(map(mean, sliding_window(2, [1, 2, 3, 4])))
+    [1.5, 2.5, 3.5]
+    """
+    it = iter(seq)
+    # An efficient FIFO data structure with maximum length
+    d = collections.deque(itertools.islice(it, n), n)
+    for item in it:
+        yield tuple(d)
+        d.append(item)
+    yield tuple(d)
+
+
+def count(seq):
+    """ Count the number of items in seq
+
+    Like the builtin ``len`` but works on lazy sequencies.
+
+    Not to be confused with ``itertools.count``
+
+    See also:
+        len
+    """
+    return sum(1 for i in seq)
